@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997-2020 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2021 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -38,11 +38,12 @@ int done;
 static void
 quit(int rc)
 {
+    SDL_Quit();
     exit(rc);
 }
 
 int
-LoadSprite(char *file, SDL_Renderer *renderer)
+LoadSprite(char *file)
 {
     SDL_Surface *temp;
 
@@ -91,7 +92,7 @@ LoadSprite(char *file, SDL_Renderer *renderer)
 }
 
 void
-MoveSprites(SDL_Renderer * renderer, SDL_Texture * sprite)
+MoveSprites()
 {
     int i;
     int window_w = WINDOW_WIDTH;
@@ -135,7 +136,7 @@ void loop()
             done = 1;
         }
     }
-    MoveSprites(renderer, sprite);
+    MoveSprites();
 #ifdef __EMSCRIPTEN__
     if (done) {
         emscripten_cancel_main_loop();
@@ -157,7 +158,7 @@ main(int argc, char *argv[])
         quit(2);
     }
 
-    if (LoadSprite("icon.bmp", renderer) < 0) {
+    if (LoadSprite("icon.bmp") < 0) {
         quit(2);
     }
 
